#include "../../../include/http/pages/peermaintenancepage.h"
#include "../../../include/stringfunctions.h"
#include "../../../include/datetime.h"

#ifdef XMEM
	#include <xmem.h>
#endif

const std::string PeerMaintenancePage::GeneratePage(const std::string &method, const std::map<std::string,std::string> &queryvars)
{
	std::string content="";
	SQLite3DB::Statement st;
	std::string tempval;
	DateTime date;

	if(queryvars.find("formaction")!=queryvars.end())
	{
		if((*queryvars.find("formaction")).second=="removenotseen")
		{
			m_db->Execute("DELETE FROM tblIdentity WHERE LastSeen IS NULL;");
		}
		else if((*queryvars.find("formaction")).second=="removelastseen20")
		{
			date.SetToGMTime();
			date.Add(0,0,0,-20);
			st=m_db->Prepare("DELETE FROM tblIdentity WHERE LastSeen<?;");
			st.Bind(0,date.Format("%Y-%m-%d %H:%M:%S"));
			st.Step();
		}
		else if((*queryvars.find("formaction")).second=="removeneversent")
		{
			m_db->Execute("DELETE FROM tblIdentity WHERE IdentityID NOT IN (SELECT IdentityID FROM tblMessage GROUP BY IdentityID);");
		}
		else if((*queryvars.find("formaction")).second=="removelastseenneversent20")
		{
			date.SetToGMTime();
			date.Add(0,0,0,-20);
			st=m_db->Prepare("DELETE FROM tblIdentity WHERE IdentityID NOT IN (SELECT IdentityID FROM tblMessage GROUP BY IdentityID) AND LastSeen<?;");
			st.Bind(0,date.Format("%Y-%m-%d %H:%M:%S"));
			st.Step();
		}
		else if((*queryvars.find("formaction")).second=="removedaysago" && queryvars.find("daysago")!=queryvars.end() && (*queryvars.find("daysago")).second!="")
		{
			int tempint=10000;
			StringFunctions::Convert((*queryvars.find("daysago")).second,tempint);
			date.SetToGMTime();
			date.Add(0,0,0,-tempint);
			st=m_db->Prepare("DELETE FROM tblIdentity WHERE IdentityID NOT IN (SELECT IdentityID FROM tblMessage GROUP BY IdentityID) AND LastSeen<?;");
			st.Bind(0,date.Format("%Y-%m-%d %H:%M:%S"));
			st.Step();
		}
	}

	content+="<h2>Peer Maintenance</h2>";
	content+="<p class=\"paragraph\">Removing a peer will not remove the messages they sent, but will remove everything else about that peer, including their trust levels.</p>";
	content+="<table>";
	content+="<tr><th colspan=\"3\">Stats</th></tr>";

	content+="<tr>";
	st=m_db->Prepare("SELECT COUNT(*) FROM tblIdentity;");
	st.Step();
	st.ResultText(0,tempval);
	content+="<td>"+tempval+"</td>";
	content+="<td>known peers</td>";
	content+="</tr>";

	content+="<tr>";
	st=m_db->Prepare("SELECT COUNT(*) FROM tblIdentity WHERE LastSeen IS NULL;");
	st.Step();
	st.ResultText(0,tempval);
	content+="<td>"+tempval+"</td>";
	content+="<td>never seen</td>";
	content+="<td>";
	content+="<form name=\"frmremove\" method=\"POST\">";
	content+="<input type=\"hidden\" name=\"formaction\" value=\"removenotseen\">";
	content+="<input type=\"submit\" value=\"Remove\">";
	content+="</form>";
	content+="</td>";
	content+="</tr>";

	date.SetToGMTime();
	date.Add(0,0,0,-20);
	st=m_db->Prepare("SELECT COUNT(*) FROM tblIdentity WHERE LastSeen<?;");
	st.Bind(0,date.Format("%Y-%m-%d %H:%M:%S"));
	st.Step();
	st.ResultText(0,tempval);
	content+="<tr>";
	content+="<td>"+tempval+"</td>";
	content+="<td>last seen more than 20 days ago</td>";
	content+="<td>";
	content+="<form name=\"frmremove\" method=\"POST\">";
	content+="<input type=\"hidden\" name=\"formaction\" value=\"removelastseen20\">";
	content+="<input type=\"submit\" value=\"Remove\">";
	content+="</form>";
	content+="</td>";
	content+="</tr>";

	st=m_db->Prepare("SELECT COUNT(*) FROM tblIdentity LEFT JOIN tblMessage ON tblIdentity.IdentityID=tblMessage.IdentityID WHERE tblMessage.IdentityID IS NULL;");
	st.Step();
	st.ResultText(0,tempval);
	content+="<tr>";
	content+="<td>"+tempval+"</td>";
	content+="<td>never sent a message</td>";
	content+="<td>";
	content+="<form name=\"frmremove\" method=\"POST\">";
	content+="<input type=\"hidden\" name=\"formaction\" value=\"removeneversent\">";
	content+="<input type=\"submit\" value=\"Remove\">";
	content+="</form>";
	content+="</td>";
	content+="</tr>";

	date.SetToGMTime();
	date.Add(0,0,0,-20);
	st=m_db->Prepare("SELECT COUNT(*) FROM tblIdentity LEFT JOIN tblMessage ON tblIdentity.IdentityID=tblMessage.IdentityID WHERE tblMessage.IdentityID IS NULL AND tblIdentity.LastSeen<?;");
	st.Bind(0,date.Format("%Y-%m-%d %H:%M:%S"));
	st.Step();
	st.ResultText(0,tempval);
	content+="<tr>";
	content+="<td>"+tempval+"</td>";
	content+="<td>last seen more than 20 days ago and never sent a message</td>";
	content+="<td>";
	content+="<form name=\"frmremove\" method=\"POST\">";
	content+="<input type=\"hidden\" name=\"formaction\" value=\"removelastseenneversent20\">";
	content+="<input type=\"submit\" value=\"Remove\">";
	content+="</form>";
	content+="</td>";
	content+="</tr>";

	content+="<tr>";
	content+="<td><form name=\"frmdelete\" method=\"POST\"><input type=\"hidden\" name=\"formaction\" value=\"removedaysago\"></td>";
	content+="<td>last seen <input type=\"text\" name=\"daysago\" size=\"2\"> days ago</td>";
	content+="<td><input type=\"submit\" value=\"Remove\"></form></td>";
	content+="</tr>";

	content+="</table>";

	return "HTTP/1.1 200 OK\r\nContent-Type: text/html\r\n\r\n"+StringFunctions::Replace(m_template,"[CONTENT]",content);
}

const bool PeerMaintenancePage::WillHandleURI(const std::string &uri)
{
	if(uri.find("peermaintenance.")!=std::string::npos)
	{
		return true;
	}
	else
	{
		return false;
	}
}
